import Flutter
import UIKit
import Kvalifika

extension UIViewController {
    func topMostViewController() -> UIViewController {
        if self.presentedViewController == nil {
            return self
        }
        if let navigation = self.presentedViewController as? UINavigationController {
            return navigation.visibleViewController!.topMostViewController()
        }
        if let tab = self.presentedViewController as? UITabBarController {
            if let selectedTab = tab.selectedViewController {
                return selectedTab.topMostViewController()
            }
            return tab.topMostViewController()
        }
        return self.presentedViewController!.topMostViewController()
    }
}

extension UIApplication {
    func topMostViewController() -> UIViewController? {
        return UIWindow.key!.rootViewController?.topMostViewController()
    }
}

extension UIWindow {
    static var key: UIWindow? {
        if #available(iOS 13, *) {
            return UIApplication.shared.windows.first { $0.isKeyWindow }
        } else {
            return UIApplication.shared.keyWindow
        }
    }
}


public class SwiftKvalifikaSdkPlugin: NSObject, FlutterPlugin {

    var channel: FlutterMethodChannel?
    
    init(channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    let ch = FlutterMethodChannel(name: "kvalifika_sdk", binaryMessenger: registrar.messenger())
    let instance = SwiftKvalifikaSdkPlugin(channel: ch)
    registrar.addMethodCallDelegate(instance, channel: ch)
  }
    
    
    private func getLocale(locale: String) -> String {
        if(locale == "EN") {
            return KvalifikaSDKLocale.EN.rawValue
        } else if (locale == "GE") {
            return KvalifikaSDKLocale.GE.rawValue
        } else {
            return KvalifikaSDKLocale.EN.rawValue
        }
    }
    
    private func initializeSDK(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let args = call.arguments as? Dictionary<String, Any>,
            let appId = args["appId"] as? String,
            let locale = args["locale"] as? String,
            let logo = args["logo"] as? String,
            let documentIcon = args["documentIcon"] as? String,
            let cancelIcon = args["cancelIcon"] as? String,
            let activeFlashIcon = args["activeFlashIcon"] as? String,
            let inactiveFlashIcon = args["inactiveFlashIcon"] as? String,
            let development = args["development"] as? Bool {
            
            func onInitialize() {
                channel?.invokeMethod("sdkOnInitialize", arguments: "")
            }
                
            func onStart(sessionId: String) {
                channel?.invokeMethod("sdkOnStart", arguments: sessionId)
            }
            
            func onFinish(sessionId: String) {
                channel?.invokeMethod("sdkOnFinish", arguments: sessionId)
            }
                
            func onError(error: KvalifikaSDKError) {
                channel?.invokeMethod("sdkOnError", arguments: error.rawValue + "___")
            }

          KvalifikaSDK.initialize(
                    AppID: appId,
                    Locale:  getLocale(locale: locale),
                    Logo: logo,
                    DocumentIcon: documentIcon,
                    CancelIcon: cancelIcon,
                    ActiveFlashIcon: activeFlashIcon,
                    InactiveFlashIcon: inactiveFlashIcon,
                    onInitialize: onInitialize,
                    onStart: onStart,
                    onFinish: onFinish,
                    onError: onError,
                    Development: development
          )
          } else {
            result(FlutterError.init(code: "bad args", message: nil, details: nil))
          }
    }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "sdkInitialize":
        initializeSDK(call, result: result)
        
    case "sdkStartSession":
        let vc = UIApplication.shared.topMostViewController()
        let _ = KvalifikaSDK.startSession(onViewController: vc!)
    default:
      result("iOS " + UIDevice.current.systemVersion)
    }
  }
}
